@@ -1,14 +1,15 @@
 /*****************************************************************************
- *                                  _   _ ____  _     
- *  Project                     ___| | | |  _ \| |    
- *                             / __| | | | |_) | |    
- *                            | (__| |_| |  _ <| |___ 
+ *                                  _   _ ____  _
+ *  Project                     ___| | | |  _ \| |
+ *                             / __| | | | |_) | |
+ *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: simplepost.c,v 1.1 2002/06/19 12:30:12 bagder Exp $
+ * $Id: simplepost.c,v 1.4 2008-05-22 21:20:09 danf Exp $
  */
 
 #include <stdio.h>
+#include <string.h>
 #include <curl/curl.h>
 
 int main(void)
@@ -16,7 +17,7 @@ int main(void)
   CURL *curl;
   CURLcode res;
 
-  char *postthis="moo mooo moo moo";
+  static const char *postthis="moo mooo moo moo";
 
   curl = curl_easy_init();
   if(curl) {
@@ -25,7 +26,7 @@ int main(void)
 
     /* if we don't provide POSTFIELDSIZE, libcurl will strlen() by
        itself */
-    curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, strlen(postthis));
+    curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, (long)strlen(postthis));
 
     res = curl_easy_perform(curl);
 
