@@ -601,6 +601,9 @@
 /* Define to the type of arg 5 for `select'. */
 #define SELECT_TYPE_ARG5 (struct timeval *)
 
+/* The size of an `off_t', as computed by sizeof. */
+#define SIZEOF_OFF_T 8
+
 /* The size of a `size_t', as computed by sizeof. */
 #define SIZEOF_SIZE_T 8
 
@@ -665,9 +668,6 @@
 /* Define to `unsigned' if <sys/types.h> does not define. */
 /* #undef size_t */
 
-/* Type to use in place of socklen_t when system does not provide it. */
-/* #undef socklen_t */
-
 /* the signed version of size_t */
 /* #undef ssize_t */
 
