@@ -7,7 +7,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2006, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2009, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -20,7 +20,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: multiif.h,v 1.7 2007-04-10 20:46:40 bagder Exp $
+ * $Id: multiif.h,v 1.10 2009-06-10 02:49:43 yangtse Exp $
  ***************************************************************************/
 
 /*
@@ -30,7 +30,7 @@ void Curl_expire(struct SessionHandle *data, long milli);
 
 void Curl_multi_rmeasy(void *multi, CURL *data);
 
-bool Curl_multi_canPipeline(struct Curl_multi* multi);
+bool Curl_multi_canPipeline(const struct Curl_multi* multi);
 void Curl_multi_handlePipeBreak(struct SessionHandle *data);
 
 /* the write bits start at bit 16 for the *getsock() bitmap */
@@ -44,4 +44,13 @@ void Curl_multi_handlePipeBreak(struct SessionHandle *data);
 /* set the bit for the given sock number to make the bitmap for readable */
 #define GETSOCK_READSOCK(x) (1 << (x))
 
+#ifdef DEBUGBUILD
+ /*
+  * Curl_multi_dump is not a stable public function, this is only meant to
+  * allow easier tracking of the internal handle's state and what sockets
+  * they use. Only for research and development DEBUGBUILD enabled builds.
+  */
+void Curl_multi_dump(const struct Curl_multi *multi_handle);
+#endif
+
 #endif /* __MULTIIF_H */
