@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2009, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,12 +18,13 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: os400sys.c,v 1.6 2008-09-24 12:22:17 yangtse Exp $
+ * $Id: os400sys.c,v 1.8 2009-05-18 10:46:17 patrickm Exp $
  *
  ***************************************************************************/
 
 /* OS/400 additional support. */
 
+#include "curlbuild.h"
 #include "config-os400.h"       /* Not setup.h: we only need some defines. */
 
 #include <sys/types.h>
@@ -237,9 +238,9 @@ buffer_undef(localkey_t key, long size)
 
 
 int
-Curl_getnameinfo_a(const struct sockaddr * sa, socklen_t salen,
-              char * nodename, socklen_t nodenamelen,
-              char * servname, socklen_t servnamelen,
+Curl_getnameinfo_a(const struct sockaddr * sa, curl_socklen_t salen,
+              char * nodename, curl_socklen_t nodenamelen,
+              char * servname, curl_socklen_t servnamelen,
               int flags)
 
 {
