@@ -7,7 +7,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2009, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -20,7 +20,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: setup.h,v 1.58 2008-09-05 01:27:24 yangtse Exp $
+ * $Id: setup.h,v 1.61 2009-07-14 13:25:14 gknauf Exp $
  ***************************************************************************/
 
 #define CURL_NO_OLDIES
@@ -39,7 +39,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "curl_config.h"
 #else
 
 #ifdef WIN32
@@ -64,10 +64,6 @@
 
 #ifdef TPF
 #include "config-tpf.h"
-/* change which select is used for the curl command line tool */
-#define select(a,b,c,d,e) tpf_select_bsd(a,b,c,d,e)
-/* and turn off the progress meter */
-#define CONF_DEFAULT (0|CONF_NOPROGRESS)
 #endif
 
 #endif /* HAVE_CONFIG_H */
@@ -128,6 +124,13 @@
 #  endif
 #endif
 
+#ifdef TPF
+#  include <sys/socket.h>
+   /* change which select is used for the curl command line tool */
+#  define select(a,b,c,d,e) tpf_select_bsd(a,b,c,d,e)
+   /* and turn off the progress meter */
+#  define CONF_DEFAULT (0|CONF_NOPROGRESS)
+#endif
 
 #include <stdio.h>
 
@@ -161,9 +164,6 @@ int fileno( FILE *stream);
 #ifdef word
 #undef word
 #endif
-#ifndef HAVE_LIMITS_H
-#define HAVE_LIMITS_H /* we have limits.h */
-#endif
 #define DIR_CHAR      "/"
 #define DOT_CHAR      "_"
 #else
