@@ -7,7 +7,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -20,9 +20,9 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: writeout.h,v 1.4 2002/09/03 11:53:01 bagder Exp $
+ * $Id: writeout.h,v 1.7 2008-04-17 00:45:34 danf Exp $
  ***************************************************************************/
 
-void ourWriteOut(CURL *curl, char *out);
+void ourWriteOut(CURL *curl, const char *out);
 
 #endif
