@@ -6,7 +6,7 @@
 #                            | (__| |_| |  _ <| |___
 #                             \___|\___/|_| \_\_____|
 #
-# Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
+# Copyright (C) 1998 - 2009, Daniel Stenberg, <daniel@haxx.se>, et al.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
@@ -19,7 +19,7 @@
 # This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 # KIND, either express or implied.
 #
-# $Id: runtests.pl,v 1.315 2008-11-19 21:56:11 bagder Exp $
+# $Id: runtests.pl,v 1.330 2009-08-11 21:48:59 bagder Exp $
 ###########################################################################
 
 # Experimental hooks are available to run tests remotely on machines that
@@ -112,6 +112,9 @@ my $SOCKSPORT; # SOCKS4/5 port
 
 my $srcdir = $ENV{'srcdir'} || '.';
 my $CURL="../src/curl"; # what curl executable to run on the tests
+my $VCURL=$CURL;   # what curl binary to use to verify the servers with
+                   # VCURL is handy to set to the system one when the one you
+                   # just built hangs or crashes and thus prevent verification
 my $DBGCURL=$CURL; #"../src/.libs/curl";  # alternative for debugging
 my $LOGDIR="log";
 my $TESTDIR="$srcdir/data";
@@ -149,8 +152,8 @@ my $SOCKSPIDFILE=".socks.pid";
 my $perl="perl -I$srcdir";
 my $server_response_maxtime=13;
 
-# this gets set if curl is compiled with debugging:
-my $curl_debug=0;
+my $debug_build=0; # curl built with --enable-debug
+my $curl_debug=0;  # curl built with --enable-curldebug (memory tracking)
 my $libtool;
 
 # name of the file that the memory debugging creates:
@@ -181,6 +184,7 @@ my $has_ipv6;    # set if libcurl is built with IPv6 support
 my $has_libz;    # set if libcurl is built with libz support
 my $has_getrlimit;  # set if system has getrlimit()
 my $has_ntlm;    # set if libcurl is built with NTLM support
+my $has_charconv;# set if libcurl is built with CharConv support
 
 my $has_openssl; # built with a lib using an OpenSSL-like API
 my $has_gnutls;  # built with GnuTLS
@@ -207,6 +211,8 @@ my $sshderror;   # for socks server, ssh daemon version error
 my $defserverlogslocktimeout = 20; # timeout to await server logs lock removal
 my $defpostcommanddelay = 0; # delay between command and postcheck sections
 
+my $testnumcheck; # test number, set in singletest sub.
+
 #######################################################################
 # variables the command line options may set
 #
@@ -587,7 +593,7 @@ sub stopserver {
 
 sub verifyhttp {
     my ($proto, $ip, $port) = @_;
-    my $cmd = "$CURL --max-time $server_response_maxtime --output $LOGDIR/verifiedserver --insecure --silent --verbose --globoff \"$proto://$ip:$port/verifiedserver\" 2>$LOGDIR/verifyhttp";
+    my $cmd = "$VCURL --max-time $server_response_maxtime --output $LOGDIR/verifiedserver --insecure --silent --verbose --globoff \"$proto://$ip:$port/verifiedserver\" 2>$LOGDIR/verifyhttp";
     my $pid;
 
     # verify if our/any server is running on this port
@@ -642,7 +648,7 @@ sub verifyftp {
     if($proto eq "ftps") {
     	$extra = "--insecure --ftp-ssl-control ";
     }
-    my $cmd="$CURL --max-time $server_response_maxtime --silent --verbose --globoff $extra\"$proto://$ip:$port/verifiedserver\" 2>$LOGDIR/verifyftp";
+    my $cmd="$VCURL --max-time $server_response_maxtime --silent --verbose --globoff $extra\"$proto://$ip:$port/verifiedserver\" 2>$LOGDIR/verifyftp";
     # check if this is our server running on this port:
     my @data=runclientoutput($cmd);
     logmsg "RUN: $cmd\n" if($verbose);
@@ -842,6 +848,7 @@ sub runhttpserver {
         # it is NOT alive
         logmsg "RUN: failed to start the HTTP$nameext server\n";
         stopserver("$pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -852,6 +859,7 @@ sub runhttpserver {
         logmsg "RUN: HTTP$nameext server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$httppid $pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -870,7 +878,7 @@ sub runhttpserver {
 # start the https server (or rather, tunnel)
 #
 sub runhttpsserver {
-    my ($verbose, $ipv6) = @_;
+    my ($verbose, $ipv6, $parm) = @_;
     my $STATUS;
     my $RUNNING;
     my $ip = $HOSTIP;
@@ -898,6 +906,7 @@ sub runhttpsserver {
     unlink($pidfile);
 
     my $flag=$debugprotocol?"-v ":"";
+    $flag .= " -c $parm" if ($parm);
     my $cmd="$perl $srcdir/httpsserver.pl $flag -p https -s \"$stunnel\" -d $srcdir -r $HTTPPORT $HTTPSPORT";
 
     my ($httpspid, $pid2) = startnew($cmd, $pidfile, 15, 0);
@@ -906,6 +915,7 @@ sub runhttpsserver {
         # it is NOT alive
         logmsg "RUN: failed to start the HTTPS server\n";
         stopservers($verbose);
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return(0,0);
     }
@@ -916,6 +926,7 @@ sub runhttpsserver {
         logmsg "RUN: HTTPS server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$httpspid $pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -984,6 +995,7 @@ sub runftpserver {
         # it is NOT alive
         logmsg "RUN: failed to start the FTP$id$nameext server\n";
         stopserver("$pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -994,6 +1006,7 @@ sub runftpserver {
         logmsg "RUN: FTP$id$nameext server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$ftppid $pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -1048,6 +1061,7 @@ sub runftpsserver {
         # it is NOT alive
         logmsg "RUN: failed to start the FTPS server\n";
         stopservers($verbose);
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return(0,0);
     }
@@ -1058,6 +1072,7 @@ sub runftpsserver {
         logmsg "RUN: FTPS server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$ftpspid $pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -1122,6 +1137,7 @@ sub runtftpserver {
         # it is NOT alive
         logmsg "RUN: failed to start the TFTP$id$nameext server\n";
         stopserver("$pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -1132,6 +1148,7 @@ sub runtftpserver {
         logmsg "RUN: TFTP$id$nameext server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$tftppid $pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -1538,9 +1555,13 @@ sub checksystem {
         }
         elsif($_ =~ /^Features: (.*)/i) {
             $feat = $1;
-            if($feat =~ /debug/i) {
-                # debug is a listed "feature", use that knowledge
+            if($feat =~ /TrackMemory/i) {
+                # curl was built with --enable-curldebug (memory tracking)
                 $curl_debug = 1;
+            }
+            if($feat =~ /debug/i) {
+                # curl was built with --enable-debug
+                $debug_build = 1;
                 # set the NETRC debug env
                 $ENV{'CURL_DEBUG_NETRC'} = "$LOGDIR/netrc";
             }
@@ -1566,6 +1587,10 @@ sub checksystem {
                 # NTLM enabled
                 $has_ntlm=1;
             }
+            if($feat =~ /CharConv/i) {
+                # CharConv enabled
+                $has_charconv=1;
+            }
         }
     }
     if(!$curl) {
@@ -1590,8 +1615,8 @@ sub checksystem {
         die "couldn't get curl's version";
     }
 
-    if(-r "../lib/config.h") {
-        open(CONF, "<../lib/config.h");
+    if(-r "../lib/curl_config.h") {
+        open(CONF, "<../lib/curl_config.h");
         while(<CONF>) {
             if($_ =~ /^\#define HAVE_GETRLIMIT/) {
                 $has_getrlimit = 1;
@@ -1619,7 +1644,7 @@ sub checksystem {
     }
 
     if(!$curl_debug && $torture) {
-        die "can't run torture tests since curl was not build with debug";
+        die "can't run torture tests since curl was not built with curldebug";
     }
 
     # curl doesn't list cryptographic support separately, so assume it's
@@ -1638,7 +1663,8 @@ sub checksystem {
 
     logmsg sprintf("* Server SSL:     %s\n", $stunnel?"ON":"OFF");
     logmsg sprintf("* libcurl SSL:    %s\n", $ssl_version?"ON":"OFF");
-    logmsg sprintf("* libcurl debug:  %s\n", $curl_debug?"ON":"OFF");
+    logmsg sprintf("* debug build:    %s\n", $debug_build?"ON":"OFF");
+    logmsg sprintf("* track memory:   %s\n", $curl_debug?"ON":"OFF");
     logmsg sprintf("* valgrind:       %s\n", $valgrind?"ON":"OFF");
     logmsg sprintf("* HTTP IPv6       %s\n", $http_ipv6?"ON":"OFF");
     logmsg sprintf("* FTP IPv6        %s\n", $ftp_ipv6?"ON":"OFF");
@@ -1732,6 +1758,10 @@ sub singletest {
     my %feature;
     my $cmd;
 
+    # copy test number to a global scope var, this allows
+    # testnum checking when starting test harness servers.
+    $testnumcheck = $testnum;
+
     # load the test case file definition
     if(loadtest("${TESTDIR}/test${testnum}")) {
         if($verbose) {
@@ -1771,7 +1801,7 @@ sub singletest {
             }
         }
         elsif($f eq "netrc_debug") {
-            if($curl_debug) {
+            if($debug_build) {
                 next;
             }
         }
@@ -1822,19 +1852,6 @@ sub singletest {
         last;
     }
 
-    my $dbghosttype=join(' ', runclientoutput("uname -a"));
-    if(($dbghosttype =~ /SMP PREEMPT/) && ($dbghosttype =~ /i686 GNU/)) {
-        if(!$curl_debug) {
-            if(($testnum != 1)   && ($testnum != 100) &&
-               ($testnum != 500) && ($testnum != 507) &&
-               ($testnum != 517) && ($testnum != 534) &&
-               ($testnum != 558) && ($testnum != 559) &&
-               ($testnum != 557) && ($testnum != 1013)) {
-                $why = "debugging icc build";
-            }
-        }
-    }
-
     if(!$why) {
         my @keywords = getpart("info", "keywords");
 	my $match;
@@ -2380,7 +2397,8 @@ sub singletest {
     }
     else {
         if(!$short) {
-            printf "\ncurl returned $cmdres, %d was expected\n", $errorcode;
+            printf("\n%s returned $cmdres, %d was expected\n",
+                   (!$tool)?"curl":$tool, $errorcode);
         }
         logmsg " exit FAILED\n";
         return 1;
@@ -2524,8 +2542,10 @@ sub startservers {
     my @what = @_;
     my ($pid, $pid2);
     for(@what) {
-        my $what = lc($_);
+        my (@whatlist) = split(/\s+/,$_);
+        my $what = lc($whatlist[0]);
         $what =~ s/[^a-z0-9-]//g;
+
         if($what eq "ftp") {
             if(!$run{'ftp'}) {
                 ($pid, $pid2) = runftpserver("", $verbose);
@@ -2627,8 +2647,8 @@ sub startservers {
                 printf ("* pid http => %d %d\n", $pid, $pid2) if($verbose);
                 $run{'http'}="$pid $pid2";
             }
-            if(!$run{'https'}) {
-                ($pid, $pid2) = runhttpsserver($verbose);
+            if(1 || !$run{'https'}) {  # QD to restart always conf file may change
+                ($pid, $pid2) = runhttpsserver($verbose,"",$whatlist[1]);
                 if($pid <= 0) {
                     return "failed starting HTTPS server (stunnel)";
                 }
@@ -2726,6 +2746,7 @@ sub serverfortest {
     for (@what) {
 	my $proto = lc($_);
 	chomp $proto;
+        $proto =~ s/\s.*//g;  # take first word
 	if (! grep /^$proto$/, @protocols) {
 	    if (substr($proto,0,5) ne "socks") {
 		    return "curl lacks $proto support";
@@ -2808,7 +2829,7 @@ while(@ARGV) {
         # keep stdout and stderr files after tests
         $keepoutfiles=1;
     }
-    elsif($ARGV[0] eq "-h") {
+    elsif(($ARGV[0] eq "-h") || ($ARGV[0] eq "--help")) {
         # show help text
         print <<EOHELP
 Usage: runtests.pl [options] [test selection(s)]
